MSP430 Base Station
Copyright (C) 2007 Joseph Bosman, Steve Olivieri, Ipek Ozil, Brandon Steacy

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA


#include "basestation.h"

#ifdef __IAR_SYSTEMS_ICC__
 #pragma vector=PORT1_VECTOR
 __interrupt void em260 (void)
#else
 interrupt (PORT1_VECTOR) em260(void)
#endif
{
	nHOST_INT = 1;
	//if(!potentialCallback)
		potentialCallback++;
	P1IFG &= ~0x08;
}

#ifdef __IAR_SYSTEMS_ICC__
 #pragma vector=USART0RX_VECTOR
 __interrupt void usart0_rx (void)
#else
 interrupt (USART0RX_VECTOR) usart0_rx(void)
#endif
{
	pcBuff[pcI] = U0RXBUF;
	if(pcBuff[pcI] == 0x0D)
		pcINT = 1;
	++pcI;
}

void delay(unsigned int time)
{
	for( ; time > 0; time--)
	{
		nop();
		nop();
	}
}

unsigned char charToHex(unsigned char* buff)
{
	return ((unsigned char)(((int)(buff[0] - (buff[0] > '9' ? 55 : 48)) * 16)
		+ (int)(buff[1] - (buff[1] > '9' ? 55 : 48))));
}

void initUART0(void)
{
	/* Use P3.4 and P3.5 for UART0 */
	P3SEL |= 0x30;

	/* P3.4 must be OUT (TXD), and P3.5 (RXD) must be IN */
	P3DIR |= 0x10;
	P3DIR &= ~0x20;

	/* Put UART0 in a reset state. */
	U0CTL |= SWRST;

	/* Enable USART0 TXD/RXD. */
	ME1 |= UTXE0 + URXE0;

	/* Set the UART parameters to:
	 * 	8-bit character
	 * 	1 stop bit
	 * 	no parity
	 */
	U0CTL |= CHAR;

	/* UCLK = SMCLK */
	U0TCTL |= SSEL1;

	/* Set the baud rate for UART0 to ~38400 bps. */
	UBR00 = 0x68;
	UBR10 = 0x00;
	
	/* Modulation. */
	U0MCTL = 0x4;
	
	/* Initialize UART0 state machine. */
	U0CTL &= ~SWRST;

	/* Wait for the Tx buffer to be ready. */
	while(!(IFG1 & UTXIFG0));

	/* Enable the UART0 RX interrupt. */
	IE1 |= URXIE0;
}

void initSPI1(void)
{
	/* Use pins 5.1-5.3 for USART1. */
	P5SEL |= 0x0E;

	/* P5.1 (MOSI) and P5.3 (SCLK) must be OUT, P5.2 (MISO) must be IN */
	P5DIR |= 0x0A;
	P5DIR &= ~0x04;

	/* Put USART1 into a reset state. */
	U1CTL |= SWRST;

	/* Enable SPI. */
	ME2 |= USPIE1;

	/* Set USART parameters to:
	 *	8-bit character
	 *	1 stop bit
	 *	no parity
	 *	SPI mode
	 *	Master mode
	 */
	U1CTL |= CHAR + SYNC + MM;

	/* BRCLK = SMCLK, 3-pin mode, phase shifting */
	U1TCTL |= SSEL1 + STC + CKPH;

	/* Set the baud rate to 500 Kbps. */
	UBR01 = 0x08;
	UBR11 = 0x00;

	/* Turn off modulation. */
	U1MCTL = 0x00;

	/* Set P5.0 to be OUT (nSSEL). */
	P5DIR |= 0x01;
	P5OUT |= 0x01;

	/* Initialize USART1 state machine. */
	U1CTL &= ~SWRST;

	/* Wait until the buffer is clear. */
	while(!(IFG2 & UTXIFG1));

	/* Enable the SPI1 RX interrupt. */
	//IE2 |= URXIE1;
}

unsigned int initEM260(void)
{
	/* Buffer for storing commands and responses */
	unsigned char buff[10];

	/* How many bytes we received */
	unsigned char i;
	i = 0;

	/* Reset the EM260 (P4.6 = 0). */
	P4OUT &= ~0x40;

	/* Wait for a while. */
	delay(0x0014);

	/* Release the reset pin. */
	P4OUT |= 0x40;

	/* Wait for EM260 to assert nHOST_INT */
	while(!nHOST_INT);
	nHOST_INT = 0;
	
	/* Now, we must transmit a specific series of commands to finish
	 * initializing the EM260.  The first command is the "version"
	 * command.  The first step is to load the "version" command (0x0A)
	 * and the frame terminator (0xA7) into the buffer.
	 */
	buff[0] = 0x0A;
	buff[1] = 0xA7;

	/* Assert nSSEL. */
	P5OUT &= ~0x01;

	/* Send the command to the EM260. */
	toEM260(buff, 2);
	//toPC(buff, 2);

	/* Clear enough buffer space to receive the response. */
	memset(buff, 0, 3);

	/* Now, receive the response. */
	i = fromEM260(buff);
	//toPC(buff, i);

	/* Deassert the nSSEL signal. */
	nHOST_INT = 0;
	P5OUT |= 0x01;

	/* If the first byte is not 0x00 and the second byte is not 0x02,
	 * there's a problem.
	 */
	if(buff[0] != 0x00 || buff[1] != 0x02)
		return 1;

	/* We must wait at least 1ms between commands. */
	delay(0x00FF);

	/* We must again transmit the "version" command. */
	buff[0] = 0x0A;
	buff[1] = 0xA7;
	P5OUT &= ~0x01;
	toEM260(buff, 2);
	//toPC(buff, 2);
	memset(buff, 0, 3);
	i = fromEM260(buff);
	//toPC(buff, i);
	nHOST_INT = 0;
	P5OUT |= 0x01;

	/* If the first byte of our response is not 0x82, there is
	 * a problem.
	 */
	if(buff[0] != 0x82)
		return 2;

	/* Again, we must delay for at least 1ms between commands. */
	delay(0x00FF);

	/* Now, transmit the "SPI status" command. */
	buff[0] = 0x0B;
	buff[1] = 0xA7;
	P5OUT &= ~0x01;
	toEM260(buff, 2);
	//toPC(buff, 2);
	memset(buff, 0, 3);
	i = fromEM260(buff);
	//toPC(buff, i);
	nHOST_INT = 0;
	P5OUT |= 0x01;

	/* If the first byte of the response is not 0xC1, there is
	 * a problem.
	 */
	if(buff[0] != 0xC1)
		return 3;
	
	/* Wait for at least 1ms. */
	delay(0x00FF);

	/* Finally, we must send the EZSP "version" command. */
	buff[0] = 0xFE;
	buff[1] = 0x04;
	buff[2] = 0x00;
	buff[3] = 0x00;
	buff[4] = 0x00;
	buff[5] = 0x02;
	buff[6] = 0xA7;
	P5OUT &= ~0x01;
	toEM260(buff, 7);
	//toPC(buff, 7);
	memset(buff, 0, 10);
	i = fromEM260(buff);
	//toPC(buff, i);
	nHOST_INT = 0;
	potentialCallback = 0;
	P5OUT |= 0x01;

	/* If bytes 5 and 6 are not each 0x02, something is wrong. */
	if(buff[5] != 0x02 || buff[6] != 0x02)
		return 4;

	/* Return successfully. */
	return 0;
}

/* Initializes the MSP430's I/O registers. */
void initValues(void)
{
	/* I/O pins aren't controlled by modules */
	P1SEL = 0x00;
	P2SEL = 0x00;
	P3SEL = 0x00;
	P4SEL = 0x00;
	P5SEL = 0x00;
	P6SEL = 0x00;

	/* All I/O pins are outputs */
	P1DIR = 0xFF;
	P2DIR = 0xFF;
	P3DIR = 0xFF;
	P4DIR = 0xFF;
	P5DIR = 0xFF;
	P6DIR = 0xFF;

	/* All output values are zero */
	P1OUT = 0x00;
	P2OUT = 0x00;
	P3OUT = 0x00;
	P4OUT = 0x00;
	P5OUT = 0x00;
	P6OUT = 0x00;

	/* Turn off P1 and P2 interrupts */
	P1IE = 0x00;
	P2IE = 0x00;

	/* All interrupts flags are reset */
	P1IFG = 0x00;
	P2IFG = 0x00;

	/* Interrupt modes are all low-to-high */
	P1IES = 0x00;
	P2IES = 0x00;
}

/* Initialize the MSP430. */
void initMSP(void)
{
	/* Use the 8MHz crystal at 4MHz. */
	BCSCTL1 &= ~XT2OFF;
	BCSCTL2 |= SELM_2 + SELS + DIVS_1;

	/* Turn the watchdog timer off. */
	WDTCTL = WDTPW | WDTHOLD;

	/* Enable interrupts. */
	_EINT();

	/* Quickly clear all I/O and interrupt registers. */
	initValues();

	/* P6.0 (LED) is OUT. */
	P6DIR |= 0x01;

	/* Init the LED to off. */
	P6OUT |= 0x01;

	/* For debugging, we also enable P2.4-2.7 as LEDs. */
	/* TODO: DEBUG */
	P2DIR |= 0xF0;
	P2OUT |= 0xF0;

	/* P4.6 (nRESET) and P4.7 (nWAKE) are OUT. */
	P4DIR |= 0xC0;
	P4OUT |= 0xC0;

	/* P1.3 (nHOST_INT) is a H->L interrupt and IN. */
	P1DIR &= ~0x08;
	P1SEL &= ~0x08;
	P1IES |= 0x08;
	P1IE |= 0x08;

	/* Initialize nHOST_INT */
	nHOST_INT = 0x00;
}

void toEM260(unsigned char* buff, unsigned int len)
{
	/* Temporary receive buffer. */
	unsigned char tmp;

	/* Loop variable */
	unsigned int i;

	for(i = 0 ; i < len; i++)
	{
		/* Wait until the Tx buffer is empty. */
		while(!(IFG2 & UTXIFG1));

		/* Transmit the next byte in the buffer. */
		U1TXBUF = *(buff+i);

		/* Receive the idle response */
		while(!(IFG2 & URXIFG1));
		tmp = U1RXBUF;
		
		/* Transmit to the PC. */
		//while(!(IFG1 & UTXIFG0));
		//U0TXBUF = *(buff+i);
	}
}

unsigned char fromEM260(unsigned char* buff)
{
	/* EZSP frame boolean */
	unsigned char ezsp;

	/* Length of the response */
	unsigned char len;

	/* Loop variable */
	unsigned char i;

	/* Initialize i to 2 to account for the first two bytes
	 * that we read.
	 */
	i = 0x02;
	ezsp = 0;
	len = 0;

	/* Transmit 0xFF and receive 0xFF to clock the SPI.  Do so until
	 * we receive a byte that is not 0xFF.  This byte is the first
	 * byte of the response.
	 */
	*buff = 0xFF;
	while(*buff == 0xFF)
	{
		while(!(IFG2 & UTXIFG1));
		U1TXBUF = 0xFF;

		while(!(IFG2 & URXIFG1));
		*buff = U1RXBUF;
	}
	
	/* Determine whether or not we're reading an EZSP frame. */
	if(*buff == 0xFE)
		ezsp = 1;

	/* Send another 0xFF to get the length of the message. */
	while(!(IFG2 & UTXIFG1));
	U1TXBUF = 0xFF;
	while(!(IFG2 & URXIFG1));
	*(buff+1) = U1RXBUF;

	/* If our response is a non-EZSP command, it may only
	 * be two bytes in length.  So, we'll just return here.
	 */
	if(*(buff+1) == 0xA7)
		return i;

	/* Else, determine the length of the message.  It it's an EZSP
	 * message, the length is the value of buff[1].  Else, the
	 * length of the message must be 0.  We add three to this
	 * value to account for the inital byte, the length byte,
	 * and the frame terminator.
	 */
	len = ezsp ? *(buff+1) : 0;
	len += 3;

	/* Now, transmit 0xFF to clock the SPI and receive the next
	 * byte of the response.  Do so until we have the entire
	 * response.
	 */
	do
	{
		/* Transmit 0xFF to the EM260. */
		while(!(IFG2 & UTXIFG1));
		U1TXBUF = 0xFF;

		/* Receive a response. */
		while(!(IFG2 & URXIFG1));
		*(buff+i) = U1RXBUF;

		/* Increment the number of bytes we've read. */
		++i;
	} while(i < len);

	/* Return the number of bytes that we received. */
	return (i);
}

void toPC(unsigned char* buff, unsigned int len)
{
	/* Loop variable */
	unsigned int i;

	for(i = 0 ; i < len; i++)
	{
		/* Wait until the Tx buffer is empty. */
		while(!(IFG1 & UTXIFG0));

		/* Transmit the next byte in the buffer. */
		U0TXBUF = *(buff+i);
	}
}

unsigned char EM260_setConfigurationValue(unsigned char* params)
{
	/* Command buffer */
	unsigned char buff[9];

	/* Set up the command buffer. */
	buff[0] = 0xFE; 	/* EZSP frame */
	buff[1] = 0x06;		/* Length of the EZSP frame */
	buff[2] = params[0];	/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x53;		/* EZSP: setConfigurationValue() command */
	buff[5] = params[1];	/* EZSP: Param: EZSP ConfigId */
	buff[6] = params[2];	/* EZSP: Param: new value (LSB) */
	buff[7] = params[3];	/* EZSP: Param: new value (MSB) */
	buff[8] = 0xA7;		/* Frame terminator */

	/* Send the command and receive the response. */
	P5OUT &= ~0x01;
	toEM260(buff, 9);
	//toPC(buff, 9);
	memset(buff, 0, 7);
	fromEM260(buff);
	//toPC(buff, 7);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;

	/* Return the result of the command. */
	return buff[5];
}

unsigned char EM260_networkInit(unsigned char sq)
{
	/* Command buffer */
	unsigned char buff[7];

	/* How many bytes we read from the 260 */
	unsigned char i;
	i = 0x00;

	/* Set up the command buffer. */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of the EZSP frame */
	buff[2] = sq;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x17;		/* EZSP: networkInit() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send the command and receive a response. */
	P5OUT &= ~0x01;
	toEM260(buff, 6);
	//toPC(buff, 6);
	memset(buff, 0, 6);
	i = fromEM260(buff);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;
	//toPC(buff, i);

	/* Return the result of the command. */
	return buff[5];
}

unsigned char EM260_leaveNetwork(unsigned char sq)
{
	/* Command buffer */
	unsigned char buff[7];

	/* Set up the command buffer. */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of the EZSP frame */
	buff[2] = sq;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x20;		/* EZSP: leaveNetwork() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send the command and receive a response. */
	P5OUT &= ~0x01;
	toEM260(buff, 6);
	//toPC(buff, 6);
	memset(buff, 0, 6);
	fromEM260(buff);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;
	//toPC(buff, 7);

	/* Return the result of the command. */
	return buff[5];
}

void EM260_formNetwork(unsigned char sq)
{
	/* Command buffer */
	unsigned char buff[19];

	/* Set up the command buffer. */
	buff[0] = 0xFE; 	/* EZSP frame */
	buff[1] = 0x10;		/* Length of the EZSP frame */
	buff[2] = sq;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x4F;		/* EZSP: scanAndFormNetwork() command */
	buff[5] = 0x00;		/* EZSP: Param: Channel Mask (LSB) */
	buff[6] = 0xF8;		/* EZSP: Param: Channel Mask */
	buff[7] = 0xFF;		/* EZSP: Param: Channel Mask */
	buff[8] = 0x07;		/* EZSP: Param: Channel Mask (MSB) */
	buff[9] = 0x03;		/* EZSP: Param: Radio Tx Power (dBm) */
	buff[10] = 0x1F;	/* EZSP: Param: Extended PAN ID (LSB) */
	buff[11] = 0x12;	/* EZSP: Param: Extended PAN ID */
	buff[12] = 0x02;	/* EZSP: Param: Extended PAN ID */
	buff[13] = 0xCA;	/* EZSP: Param: Extended PAN ID */
	buff[14] = 0x48;	/* EZSP: Param: Extended PAN ID */
	buff[15] = 0x25;	/* EZSP: Param: Extended PAN ID */
	buff[16] = 0x33;	/* EZSP: Param: Extended PAN ID */
	buff[17] = 0xF1;	/* EZSP: Param: Extended PAN ID (MSB) */
	buff[18] = 0xA7;	/* Frame terminator */

	/* Send the command. */
	P5OUT &= ~0x01;
	toEM260(buff, 19);
	nHOST_INT = 0;
	potentialCallback--;
	delay(0x0FFF);
	//toPC(buff, 19);
	P5OUT |= 0x01;
}

unsigned char EM260_permitJoining(unsigned char sq)
{
	/* Command buffer */
	unsigned char buff[7];

	/* Set up the command buffer. */
	buff[0] = 0xFE; 	/* EZSP frame */
	buff[1] = 0x04;		/* Length of the EZSP frame */
	buff[2] = sq;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x22;		/* EZSP: permitJoining() command */
	buff[5] = 0xFF;		/* EZSP: Param: duration (infinite) */
	buff[6] = 0xA7;		/* Frame terminator */

	/* Send the command and receive the response. */
	P5OUT &= ~0x01;
	toEM260(buff, 7);
	//toPC(buff, 7);
	memset(buff, 0, 7);
	fromEM260(buff);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;
	//toPC(buff, 7);

	/* Return the result of the command. */
	return buff[5];
}

void EM260_getNetworkParameters(unsigned char* buff, unsigned char sq)
{
	unsigned char i;

	/* Set up the command buffer. */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of the EZSP frame */
	buff[2] = sq;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x28;		/* EZSP: getNetworkParameters() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send the command and receive a response. */
	P5OUT &= ~0x01;
	toEM260(buff, 6);
	//toPC(buff, 6);
	memset(buff, 0 , 6);
	i = fromEM260(buff);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;
	//toPC(buff, i);
}

void EM260_lookupNodeID(unsigned char* EUI, unsigned char* nodeID, unsigned char sq)
{
	/* Command bufer */
	unsigned char command[14];

	/* Set up the command buffer. */
	command[0] = 0xFE;	/* EZSP frame */
	command[1] = 0x0B;	/* Length of the EZSP frame */
	command[2] = sq;	/* EZSP: sequence number */
	command[3] = 0x00;	/* EZSP: command with no sleeping */
	command[4] = 0x60;	/* EZSP: lookupNodeIdByEui64() command */
	command[5] = EUI[7];	/* EZSP: Param: EUI64 (LSB) */
	command[6] = EUI[6];	/* EZSP: Param: EUI64 */
	command[7] = EUI[5];	/* EZSP: Param: EUI64 */
	command[8] = EUI[4];	/* EZSP: Param: EUI64 */
	command[9] = EUI[3];	/* EZSP: Param: EUI64 */
	command[10] = EUI[2];	/* EZSP: Param: EUI64 */
	command[11] = EUI[1];	/* EZSP: Param: EUI64 */
	command[12] = EUI[0];	/* EZSP: Param: EUI64 (MSB) */
	command[13] = 0xA7;	/* Frame terminator */

	/* Send the command and receive a response. */
	P5OUT &= ~0x01;
	toEM260(command, 14);
	//toPC(command, 14);
	memset(command, 0, 14);
	fromEM260(command);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;
	//toPC(command, 8);
	
	/* Get the NodeID and put it in the return buffer. */
	memcpy(nodeID, command+5, 2);
}

void handleCallback(unsigned char* buff, unsigned char sq)
{
	/* The EUI64 of an incoming message's sender */
	unsigned char EUI[8];

	/* The NodeID of an incoming message's sender */
	unsigned char nodeID[2];

	/* Buffer for outgoing messages */
	unsigned char buff2[136];

	/* Loop variable and counter */
	unsigned char i;

	/* Determine which callback we got. */
	if(buff[4] == 0x00 || buff[4] == 0x07)
	{
		/* There were no callbacks, or we have an error.
		 * Either way, we can ignore it.
		 */
		return;
	}
	else if(buff[4] == 0x62)
	{
		/* We have an incoming EUI64, which will be
		 * followed by an incoming unicast.  We need
		 * to store this address, format the message,
		 * and send everything to the PC.
		 *
		 * We read the EUI64 in backwards because it
		 * was sent to us in a little endian format
		 * and the PC program wants it in big endian
		 * format.
		 */
		for(i = 0; i < 8; i++)
		{
			EUI[i] = buff[13-i];
		}
		
		/* Now we clear the buffer, wait at least 1ms,
		 * and get the actual message from the EM260.
		 */
		memset(buff, 0, 136);
		while(buff[4] != 0x45)
		{
			memset(buff, 0, 136);
			delay(0x00FF);
			i = EM260_callback(buff, sq);
		}

		/* The 24th byte is the length of the message. */
		i = *(buff+23);
		
		/* Get the sender's NodeID so we can reply to them. */
		memcpy(nodeID, buff+24, 2);

		/* Now, we must format our string to look like
		 * the one the PC is expecting.
		 */
		snprintf(buff2, 26, "\r\nUCAST:%.2x%.2x%.2x%.2x%.2x%.2x%.2x%.2x=",
			(int)EUI[0], (int)EUI[1], (int)EUI[2], (int)EUI[3],
			(int)EUI[4], (int)EUI[5], (int)EUI[6], (int)EUI[7]);

		memcpy(buff2+25, buff+26, i-4);
		memcpy(buff2+21+i, "\r\n", 2);
		
		/* Now we send the message to the PC. */
		toPC(buff2, 23+i);
		potentialCallback--;

		/* If the message was an IMAGE_READY signal, we need to store
		 * the node's ID and set imageReady so that we can reply
		 * later.
		 */
		if(strncmp(buff2+25, "IMAGE_READY", 11) == 0)
		{
			if(!imageReady)
			{
				imageReady = 1;
				memcpy(irID, nodeID, 2);
			}
		}

		/* Finally, we sent a message back to the node to let it know
		 * that it's okay to send another message.
		 */
		memset(buff, 0, 136);
		strncpy(buff, "OK", 3);
		EM260_sendUnicast(nodeID, buff, 2, sq++, 0);
	}
	else
	{
		/* We can ignore all other callbacks. */
		return;
	}
}

unsigned char EM260_callback(unsigned char* buff, unsigned char sq)
{
	/* How many bytes the response is */
	unsigned char i;

	/* Set up the command buffer. */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of the EZSP frame */
	buff[2] = sq;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x06;		/* EZSP: callback() command */
	buff[5] = 0xA7;		/* Frame terminator */
	
	/* Send the command and receive a response. */
	P5OUT &= ~0x01;
	toEM260(buff, 6);
	//toPC(buff, 6);
	memset(buff, 0, 6);
	//delay(0x0FFF);
	i = fromEM260(buff);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;
	//toPC(buff, i);

	/* Return how many bytes the callback response is. */
	return i;
}

unsigned char EM260_sendUnicast(unsigned char* addr, unsigned char* message, unsigned char len, unsigned char sq, unsigned char sq2)
{
	/* Command buffer */
	unsigned char command[22+len];

	/* Loop variable and counter */
	unsigned int i;

	/* Set up the command buffer. */
	command[0] = 0xFE;	/* EZSP frame */
	command[1] = 19 + len;	/* Length of the EZSP frame */
	command[2] = sq;	/* EZSP: sequence number */
	command[3] = 0x00;	/* EZSP: command with no sleeping */
	command[4] = 0x34;	/* EZSP: sendUnicast() command */
	command[5] = 0x00;	/* EZSP: Param: message type (direct) */
	command[6] = addr[0];	/* EZSP: Param: nodeID (LSB) */
	command[7] = addr[1];	/* EZSP: Param: nodeID (MSB) */
	command[8] = 0x00;	/* EZSP: Param: profileID (LSB) */
	command[9] = 0x00;	/* EZSP: Param: profileID (MSB) */
	command[10] = 0x00;	/* EZSP: Param: clusterID (LSB) */
	command[11] = 0x00;	/* EZSP: Param: clusterID (MSB) */
	command[12] = 0x01;	/* EZSP: Param: source endPoint */
	command[13] = 0x01;	/* EZSP: Param: destination endPoint */
	command[14] = 0x40;	/* EZSP: Param: APS options (LSB) */
	command[15] = 0x05;	/* EZSP: Param: APS options (MSB) */
	command[16] = 0x00;	/* EZSP: Param: groupID (LSB) */
	command[17] = 0x00;	/* EZSP: Param: groupID (MSB) */
	command[18] = sq2;	/* EZSP: Param: sequence number */
	command[19] = 0x64;	/* EZSP: Param: message tag */
	command[20] = len;	/* EZSP: Param: message length */
	for(i = 0; i < len; i++)
	{
		command[i+21] = message[i];	/* EZSP: Param: message */
	}
	command[21+len] = 0xA7;	/* Frame terminator */

	/* Send the command and receive a response. */
	P5OUT &= ~0x01;
	toEM260(command, 22+len);
	//toPC(command, 22+len);
	memset(command, 0, 22+len);
	i = fromEM260(command);
	//toPC(command, i);
	nHOST_INT = 0;
	potentialCallback--;
	P5OUT |= 0x01;

	/* Return the result of the command. */
	return command[5];
}

void main(void)
{
	/* Message buffers */
	unsigned char buff1[136];
	unsigned char buff2[136];
	
	/* Our network information */
	unsigned char channel;
	unsigned char panID[2];

	/* Sequence number for SPI transactions */
	unsigned char sq;

	/* Sequence number for unicast messages. */
	unsigned char sq2;

	/* Length of unicast messages, or number of bytes read with 
	 * fromEM260() or fromPC()
	 */
	unsigned char len;

	/* The status of the network (0 = DOWN, 1 = UP) */
	unsigned char netStat;

	/* Status returned from EM260 command */
	unsigned char status;
	
	status = 0x01;
	sq2 = 0x00;
	imageReady = 0;

	/* Initialize hardware. */
	initMSP();
	initUART0();
	initSPI1();
	while(status != 0x00)
		status = initEM260();
	
	/* Initialize the sequence number to 1 since we used sq = 0 in
	 * the initEM260() call.
	 */
	sq = 0x01;

	/* Configure the EM260 to function properly.  The fist configuration
	 * call turns on boost mode (EZSP_CONFIG_TX_POWER_MODE while the
	 * second enables meissage relaying (EZSP_CONFIG_DISABLE_RELAY).
	 * Finally, we turn off security.
	 */
	delay(0xFFFF);
	buff1[0] = sq++;
	buff1[1] = 0x17;
	buff1[2] = 0x01;
	buff1[3] = 0x00;
	status = 0x01;
	while(status != 0x00)
		status = EM260_setConfigurationValue(buff1);
	memset(buff1, 0, 4);

	delay(0xFFFF);
	buff1[0] = sq++;
	buff1[1] = 0x18;
	buff1[2] = 0x00;
	buff1[3] = 0x00;
	status = 0x01;
	while(status != 0x00)
		status = EM260_setConfigurationValue(buff1);
	memset(buff1, 0, 4);

	delay(0xFFFF);
	buff1[0] = sq++;
	buff1[1] = 0x0D;
	buff1[2] = 0x00;
	buff1[3] = 0x00;
	status = 0x01;
	while(status != 0x00)
		status = EM260_setConfigurationValue(buff1);
	memset(buff1, 0, 4);

	/* Now, we make a call to networkInit().  This should be done every
	 * time we reboot, regardless of whether or not we used to be on
	 * a network.  If we are on a network after networkInit(), we
	 * must leave that network.
	 */
	delay(0xFFFF);
	status = EM260_networkInit(sq++);
	if(status != 0x93)
	{
		P2OUT &= ~0x10; /* TODO: DEBUG */
		delay(0xFFFF);
		while(!nHOST_INT);
		len = EM260_callback(buff1, sq);
		memset(buff1, 0, len);
	
		status = 0x01;
		while(status != 0x00)
		{
			delay(0xFFFF);
			status = EM260_leaveNetwork(sq++);
		}
		
		delay(0xFFFF);
		while(!nHOST_INT);
		len = EM260_callback(buff1, sq);
		memset(buff1, 0, len);
	}
	
	/* Now, we're certainly not on a network. */
	netStat = 0;
	channel = 0x00;
	panID[0] = 0x00;
	panID[1] = 0x00;

	P2OUT |= 0x10; /* TODO: DEBUG */

	/* Turn the LED on to let the adminstrator know that we're ready. */
	P6OUT &= ~0x01;
	potentialCallback = 0;

	/* TODO: DEBUG */
	P2OUT &= ~0x20;

	/* Now, we simply process input from the PC and from the EM260. */
	while(1)
	{
		/* Determine whether or not we have a command from the PC. */
		if(pcINT)
		{
			/* If so, we'll deal with it. */
			pcINT = 0;
			len = pcI;
			pcI = 0;

			/* Read the command from the PC. */
			if(strncmp(pcBuff, "AT+UCAST", 8) == 0)
			{
				/* We attempt to send the message to the
				 * NodeID that we stored when we got an
				 * IMAGE_READY signal.  We don't send any
				 * messages but IMAGE_READY_OK, so this
				 * works.  It's not optimal, but can
				 * easily be extended to support all
				 * outgoing unicasts.
				 */
				status = EM260_sendUnicast(irID, pcBuff+26, len-27, sq++, sq2++);
				if(status != 0x00)
				{
					snprintf(buff2, 11, "ERROR:%.2x\n\r",
						(int)status);
					toPC(buff2, 10);
					memset(buff2, 0, 11);
				}
				else
				{
					snprintf(buff2, 5, "OK\n\r");
					toPC(buff2, 4);
					memset(buff2, 0, 5);
					imageReady = 0;
				}

				memset(pcBuff, 0, len);
			}
			else if(strncmp(pcBuff, "AT+EN", 5) == 0)
			{
				memset(pcBuff, 0, len);

				/* If we're already joined to the network,
				 * let the PC know that everything is OK.
				 * Else, we must attempt to form the network
				 * and let the PC know the result of our
				 * attempt.
				 */
				if(netStat)
				{
					snprintf(buff2, 15, "JPAN:%.2x,%.2x%.2x\n\r",
						(int)channel, (int)panID[0],
						(int)panID[1]);
					toPC(buff2, 14);
					memset(buff2, 0, 15);
				}
				else
				{
					delay(0x0FFF);
					EM260_formNetwork(sq++);
					while(!nHOST_INT);
					P2OUT &= ~0x80; /* TODO: DEBUG */
					delay(0x0FFF);
					len = EM260_callback(buff1, sq);
					memset(buff1, 0, len);
					while(!nHOST_INT);
					P2OUT &= ~0x40; /* TODO: DEBUG */
					delay(0x0FFF);
					len = EM260_callback(buff1, sq);
					if(buff1[5] != 0x90)
					{
						snprintf(buff2, 11,
							"ERROR:%.2x\n\r",
							(int)buff1[5]);
						toPC(buff2, 10);
						memset(buff2, 0, 11);
						netStat = 0;
						P6OUT |= 0x10; /* TODO: DEBUG */
						memset(buff1, 0, len);
						continue;
					}

					delay(0x0FFF);
					status = EM260_permitJoining(sq++);
					delay(0x0FFF);
					EM260_getNetworkParameters(\
						buff2, sq++);
					channel = buff2[18];
					panID[0] = buff2[16];
					panID[1] = buff2[15];
					snprintf(buff2, 15, "JPAN:%.2x,%.2x%.2x\n\r",
						(int)channel,
						(int)panID[0],
						(int)panID[1]);
					toPC(buff2, 14);
					memset(buff2, 0, 15);
					netStat = 1;
					P2OUT &= ~0x10; /* TODO: DEBUG */
					memset(buff1, 0, len);
				}
			}
			else if(strncmp(pcBuff, "AT+DASSL", 8) == 0)
			{
				memset(pcBuff, 0, len);

				/* If we're not joined to a network, there
				 * is no reason to try to leave one.  So
				 * we just tell the PC that everything is
				 * okay.
				 */
				if(!netStat)
				{
					snprintf(buff1, 10, 
						"LeftPAN\r\n");
					toPC(buff1, 9);
					memset(buff1, 0, 10);
					continue;
				}

				/* Else, if we are joined to a network, we
				 * must now leave it and let the PC know
				 * if we were successful in doing so.
				 */
				delay(0x0FFF);
				status = EM260_leaveNetwork(sq++);
				P2OUT |= 0x10;
				delay(0x0FFF);
				if(status == 0x00)
				{
					while(!nHOST_INT);
					len = EM260_callback(buff1, sq);
					P2OUT |= 0x40;
					if(buff1[5] == 0x91)
					{	netStat = 0;
						channel = 0x00;
						panID[0] = 0x00;
						panID[1] = 0x00;
						//P2OUT |= 0x80; /* TODO: DEBUG */
						P2OUT |= 0x80;
						memset(buff1, 0, len);
						snprintf(buff1, 10,
							"LeftPAN\r\n");
						toPC(buff1, 9);
						memset(buff1, 0, 10);
					}
					else
					{
						memset(buff1, 0, len);
						snprintf(buff1, 11,
							"ERROR:%.2x\r\n",
							(int)buff1[5]);
						toPC(buff1, 10);
						memset(buff1, 0, 11);
					}
				}
				else
				{
					snprintf(buff1, 11,
						"ERROR:%.2x\r\n",
						(int)status);
					toPC(buff1, 10);
					memset(buff1, 0, 11);
				}
			}
			else
			{
				/* We ignore all other commands as they do
				 * not matter to us.
				 */
				memset(pcBuff, 0, len);
			}
		}

		/* Send a callback to the EM260 to find out if it has
		 * anything for us.  If so, we need to handle it.
		 */
		if(potentialCallback)
		{
			delay(0x00FF);
			EM260_callback(buff1, sq++);
			delay(0x00FF);
			handleCallback(buff1, sq++);
			memset(buff1, 0, 136);
			potentialCallback--;
		}
	}
}
