MSP430 Camera Node
Copyright (C) 2007 Joseph Bosman, Matthew Conway, Phong Dam, Steve Olivieri,
	Ipek Ozil, Janelle Tavares, Brandon Steacy

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA


#include "node.h"

/************* Timer A ISR *****************/
#ifdef __IAR_SYSTEMS_ICC__
  #pragma vector = TIMERA0_VECTOR
  __interrupt void Timer_A0(void)
#else
  interrupt (TIMERA0_VECTOR) Timer_A0(void)
#endif
{ 
	timeflag_a = 1;
}

/************* Timer B ISR *****************/
#ifdef __IAR_SYSTEMS_ICC__
  #pragma vector = TIMERB0_VECTOR
  __interrupt void Timer_B0(void)
#else
  interrupt (TIMERB0_VECTOR) Timer_B0(void)
#endif
{	
	timeoutFlag++;
	timeflag_b = 1;
}

/************* Port 2 ISR *****************/
#ifdef __IAR_SYSTEMS_ICC__
  #pragma vector = PORT2_VECTOR
  __interrupt void port_2(void)
#else
  interrupt (PORT2_VECTOR) port_2(void)
#endif
{
	_BIC_SR_IRQ(LPM1_bits); // Exit LPM1
	P2IFG = 0;	 // clear Port 2 interrupt flag
}

/************* UART0 RX ISR *****************/
#ifdef __IAR_SYSTEMS_ICC__
  #pragma vector = UART0RX_VECTOR	  // uart0 receive interrupt
  __interrupt void usart0_rx (void)
#else
  interrupt (UART0RX_VECTOR) usart0_rx(void)
#endif
{	
	recvbuffer[currentLen++] = RXBUF0;		 
	if (dataReady == 1)
		dummybuffer[dummyLen++] = recvbuffer[dummyRecv++];
}

#ifdef __IAR_SYSTEMS_ICC__
 #pragma vector=PORT1_VECTOR
 __interrupt void em260 (void)
#else
 interrupt (PORT1_VECTOR) em260(void)
#endif
{
	nHOST_INT = 1;
	P1IFG &= ~0x08;
}


/******************** main () ***************************
*  The system will first enter sleep mode. As soon as the
*  sensor triggers, the system will be activated and start
*  taking and storing 2 pictures onto SD card. It will 
*  then go back to sleep mode. The routine repeats.
*********************************************************/
void main( void )
{
  /* Command buffer for the EM260 */
  unsigned char buff[3];

  /* Number of pictures taken */
  unsigned char pic_counter;

  /* Staitus for EM260 functions */
  unsigned char status;
 
  /* Loop variable */
  unsigned int i;

  sq = 0x01;
  sq2 = 0x00;

  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  _BIS_SR(GIE);  // Global interrupt enable
  P6OUT |= 0x04;
  init_sys();
  config_camera_cmd();  
  LEDOn();
  init_SD();		// initialize SD card
  //waitsec(100);	// wait for sensor stablize

  /* Initialize the EM260 */
  enableSPI0();
  status = 0xFF;
  while(status != 0x00)
  {
  	wait_a();
  	status = initEM260();
  }
  blinkLED(1);
  
  /* Turn off ZigBee security. */
  buff[0] = 0x0D;
  buff[1] = 0x00;
  buff[2] = 0x00;
  do
  {
  	wait_a();
  	status = EM260_setConfigurationValue(buff);
  } while(status != 0x00);

  /* Enable message relaying. */
  buff[0] = 0x18;
  buff[1] = 0x00;
  buff[2] = 0x00;
  do
  {
  	wait_a();
	status = EM260_setConfigurationValue(buff);
  } while(status != 0x00);

  blinkLED(2);
  
  /* Join a ZigBee network. */
  EM260_networkInit();
  blinkLED(3);
  
  joinNetwork();
  blinkLED(4);
  
  /* Disable SPI and enable UART so that the camera can begin functioning. */
  disableSPI0();
  enableUART0();
  
  /* From this point on, we wait for input from the PIR sensor.  If we get it,
   * we take pictures and send them over the ZigBee network.  Forever!
   */
  while (1) 
  {
  	/* First, we put the MSP430 into sleep mode.  We remain in sleep mode
	 * unless the PIR sensor wakes us up.
	 */
	pic_counter = 0; // keep track of how many pictures taken  
	LEDOff();
	reset_sys();
	
	#ifdef __IAR_SYSTEMS_ICC__
		_BIS_SR(LPM1);   // LPM1 sleep mode enabled: MCLK and DCO are disabled
	#else			 // SMCLK and ACLK are active	 
		LPM1;
	#endif
	blinkLED(0);	 //  indicate system is activated by sensor  
	cam_prep();	  // initialize camera by sending multiple SYNC commands
	blinkLED(1);	 //  indicate camera is succesfully initialized
	 
 	/* Then, we take pictures. */
	while (pic_counter < num_pic_taken)
	{
		take_pic();	  // tell camera to take snapshot	
		get_image();	 // get image data from camera				
		 
		reset_sys();
		pic_counter++;	 
		blinkLED(3);	// done taking a picture
		waitsec(3);	 
	}
	send_cmd(cmdReset);
	send_cmd(cmdPowerOff);	 // turn off camera	
	blinkLED(4);		// done taking user-specified number of pictures
	 
	 /* Finally, we transmit our pictures over the ZigBee network. */
	disableUART0();
	enableSPI0();
	for(i = 0; i < pic_counter; i++)
	{
		transmitPicture(i);
		waitsec(10);
	}
	disableSPI0();
	enableUART0();

	/* Now we reset the number of pictures taken and point block back to
	 * block 0, as we no longer need the data on the SD card.
	 */
	numOfPic = 0;
	block = 0;
	numBlocks = 0;
  }
}


/******************* init_sys() ************************/
void init_sys(void)
{
  volatile unsigned char dco_cnt = 255;
  BCSCTL1 &= ~XT2OFF;				  // XT2on
  do							// wait for MCLK from quartz
  {
	 IFG1 &= ~OFIFG;					 // Clear OSCFault flag
	 for (dco_cnt = 0xff; dco_cnt > 0; dco_cnt--); // Time for flag to set
  }
  while ((IFG1 & OFIFG) != 0);			 // OSCFault flag still set?
  BCSCTL2 |= SELM_2 + SELS + DIVS_1;			// MCLK = XT2  SMCLK= MCLK / 2 = 4MHz
  
  P6DIR |= BIT0 + BIT2 + BIT4 + BIT5 + BIT6 + BIT7;		// Set P6.0,  P6.4 - 6.7, to output direction
  P6SEL &= ~(BIT0 + BIT2 + BIT4 + BIT5 + BIT6 + BIT7);	// Make P6.0, P.4 - 7, I/O option

  P2DIR &= ~BIT0;	  // Set P2.0 to input direction
  P2SEL &= ~BIT0;	  // Make P2.0 I/O option
  P2IES = 0x00;		 // Trigger mode: Low to high transition 
  P2IE = 0x01;	// Enable Port 2 interrupt 
  
  /* P4.0 (nRESET) and P4.1 (nWAKE) are OUT. */
  P4DIR |= 0x03;
  P4OUT |= 0x03;

  /* P1.3 (nHOST_INT) is a H->L interrupt and IN. */
  P1DIR &= ~0x08;
  P1SEL &= ~0x08;
  P1IES |= 0x08;
  P1IE |= 0x08;

  /* Turn off the LEDs */
  LED1Off();
  LED2Off();
  LED3Off();
  LED4Off();
  //P6OUT |= 0x04;
}

/****************** init_SD () ************************/
void init_SD (void)
{
  int i;
  unsigned short int success = 0;
  
  while (!success)
  {
	 runtimerb();
	 while (timeoutFlag < timeout)
	 {
		if (initMMC() == MMC_SUCCESS)	  // card found
		{
	card_state |= 1;
	success = 1;
		}
		else
		{
	//Error card not detected or rejected during writing
	card_state = 0;				  // no card
		}
	 }
	 stoptimerb();
	 timeoutFlag = 0;
	 if (card_state == 0) 
		for (i = 0; i < 3; i++)
			blinkLED(0);
  }
}

/******************* runtimera() ************************/
void runtimera(void)
{
  
  TACTL = TASSEL_2 + CNTL_0 + MC_1 + ID_3; // SMCLK, 16bit, up mode, 8 divider
  TACCR0 = 0X1388;  // 5000 SMCLK tics = 0.01 second
  TACCTL0 = CCIE;	// TACCR0 interrupt enabled
  
  /*
  TACTL = TASSEL_1 + CNTL_0 + MC_1 + ID_0; // ACLK, 16bit, up mode, 1 divider
  TACCR0 = 0x8000;  // 32768 ACLK tics = 1 second
  TACCTL0 = CCIE;	// TACCR0 interrupt enabled
  */
}

/******************* stoptimera() ************************/
void stoptimera(void)
{
  TACTL = MC_0; // stop timer
  TACCTL0 &= ~CCIE;  // TACCR0 interrupt disabled
}

/******************* runtimerb() ************************/
void runtimerb(void)
{
  TBCTL = TBSSEL_2 + CNTL_0 + MC_1 + ID_3; // SMCLK, 16bit, up mode, 8 divider
  TBCCR0 = 0xC350;  // 50000 SMCLK tics = 0.1 second
  TBCCTL0 = CCIE;	// TBCCR0 interrupt enabled
}

/******************* stoptimerb() ************************/
void stoptimerb(void)
{
  TBCTL = MC_0; // stop timer
  TBCCTL0 &= ~CCIE;  // TBCCR0 interrupt disabled
}

/***************** enableUART0() *********************/
void enableUART0(void)
{
	 P3SEL |= 0x30;		 // P3.4 , P3.5 = USART0 option select  
	 U0CTL |= SWRST;
	 ME1 |= UTXE0 + URXE0;	  // Enable USART0 TXD/RXD
	 UCTL0 |= CHAR;		// 8-bit character , NO parity, 1 stop bit
	 UCTL0 &= ~(SYNC + MM);
	 UTCTL0 |= SSEL1;	  // UCLK = SMCLK
	 UBR00 = 0x68;		  // 4Mhz/38400 ~ 104
	 UBR10 = 0x00;		  //
	 UMCTL0 = 0x4;		  // modulation
	 UCTL0 &= ~SWRST;	  // Initialize USART state machine
	 IE1 |= URXIE0;		 // Enable USART0 RX interru
}
/***************** enableUART0() *********************/
void disableUART0(void)
{
	 ME1 &= ~(UTXE0 + URXE0);	  // Disable USART0 TXD/RXD
}

/**************** enableSPI0() **********************/
void enableSPI0(void)
{
	/* Use pins 3.1-3.3 for USART1. */
	P3SEL |= 0x0E;

	/* P3.1 (MOSI) and P3.3 (SCLK) must be OUT, P3.2 (MISO) must be IN */
	P3DIR |= 0x0A;
	P3DIR &= ~0x04;

	/* Put USART0 into a reset state. */
	U0CTL |= SWRST;

	/* Enable SPI. */
	ME1 |= USPIE0;

	/* Set USART parameters to:
	 *	  8-bit character
	 *	  1 stop bit
	 *	  no parity
	 *	  SPI mode
	 *	  Master mode
	 */
	U0CTL |= CHAR + SYNC + MM;

	/* BRCLK = SMCLK, 3-pin mode, phase shifting */
	U0TCTL |= SSEL1 + STC + CKPH;

	/* Set the baud rate to 500 Kbps. */
	UBR00 = 0x08;
	UBR10 = 0x00;

	/* Turn off modulation. */
	U0MCTL = 0x00;

	/* Set P3.0 to be OUT (nSSEL). */
	P3DIR |= 0x01;
	P3OUT |= 0x01;

	/* Initialize USART0 state machine. */
	U0CTL &= ~SWRST;

	/* Wait until the buffer is clear. */
	while(!(IFG1 & UTXIFG0));
}

void disableSPI0(void)
{
	/* Disables SPI on USART0 */
	ME1 &= ~USPIE0;
}

/******************* LEDOn() ************************/
void LEDOn(void)
{
	 P6OUT &= ~BIT0; // P6.0 output = 0 (LED off)
}

/******************* LEDOff() ************************/
void LEDOff(void)
{
	 P6OUT |= BIT0;  // P6.0 output = 1 (LED off)
}

/******************* LED1On() ************************/
void LED1On(void)
{
	 P6OUT &= ~BIT6; // P6.6 output = 0 (LED off)
}

/******************* LED1Off() ************************/
void LED1Off(void)
{
	 P6OUT |= BIT6;  // P6.6 output = 1 (LED off)
}
/******************* LED2On() ************************/
void LED2On(void)
{
	 P6OUT &= ~BIT7; // P6.7 output = 0 (LED off)
}

/******************* LED2Off() ************************/
void LED2Off(void)
{
	 P6OUT |= BIT7;  // P6.7 output = 1 (LED off)
}
/******************* LED3On() ************************/
void LED3On(void)
{
	 P6OUT &= ~BIT4; // P6.4 output = 0 (LED off)
}

/******************* LED3Off() ************************/
void LED3Off(void)
{
	 P6OUT |= BIT4;  // P6.4 output = 1 (LED off)
}

/******************* LED4On() ************************/
void LED4On(void)
{
	 P6OUT &= ~BIT5; // P6.5 output = 0 (LED off)
}

/******************* LED4Off() ************************/
void LED4Off(void)
{
	 P6OUT |= BIT5;  // P6.5 output = 1 (LED off)
}

/******************* wait_a() *********************/
void wait_a(void)
{
  runtimera();
  while (!timeflag_a); 
  timeflag_a = 0;
  stoptimera();
}

/******************* wait() *********************/
void wait(void)
{
  runtimerb();
  while (!timeflag_b); 
  timeflag_b = 0;
  stoptimerb();
}
/******************* wait_a_sec () *********************/
void wait_a_sec (unsigned short int s)
{
  int i;
  for (i = 0; i < s; i++) 
	 wait_a();
}  
/******************* waitsec () *********************/
void waitsec (unsigned short int s)
{
  int i;
  for (i = 0; i < s; i++) 
	 wait();
}  

/******************* config_camera_cmd () *********************

  * 2nd element of an array is the command's ID.
  * 3rd, 4th, 5th, and 6th elements of an array are 
		 Parameter 1, 2, 3, and 4 respectively.
				
***************************************************************/
void config_camera_cmd(void)
{
  
  cmdSYNC[0] = 0xAA;
  cmdSYNC[1] = 0x0D;
  cmdSYNC[2] = 0x00;
  cmdSYNC[3] = 0x00;
  cmdSYNC[4] = 0x00;
  cmdSYNC[5] = 0x00;
  
  cmdACK[0] = 0xAA;
  cmdACK[1] = 0x0E;
  cmdACK[2] = 0x0D;
  cmdACK[3] = 0x00;
  cmdACK[4] = 0x00;
  cmdACK[5] = 0x00;
  
  cmdReset[0] = 0xAA;
  cmdReset[1] = 0x08;
  cmdReset[2] = 0x00;
  cmdReset[3] = 0x00;
  cmdReset[4] = 0x00;
  cmdReset[5] = 0xFF;
  
  cmdPowerOff[0] = 0xAA;
  cmdPowerOff[1] = 0x09;
  cmdPowerOff[2] = 0x00;
  cmdPowerOff[3] = 0x00;
  cmdPowerOff[4] = 0x00;
  cmdPowerOff[5] = 0x00;
  
  cmdInitial[0] = 0xAA;
  cmdInitial[1] = 0x01;
  cmdInitial[2] = 0x00;
  cmdInitial[3] = 0x07;	// JPEG file
  cmdInitial[4] = 0x00;
  cmdInitial[5] = resolution;	
 
  
  // package size is 262 bytes = 0x0106h
  cmdSetPackageSize[0] = 0xAA;
  cmdSetPackageSize[1] = 0x06;
  cmdSetPackageSize[2] = 0x08;
  cmdSetPackageSize[3] = 0x06;	 // Low Byte size
  cmdSetPackageSize[4] = 0x01;	 // High Byte size
  cmdSetPackageSize[5] = 0x00;
  
  // the baud rate = 38400 bps
  cmdSetBaudRate[0] = 0xAA;
  cmdSetBaudRate[1] = 0x07;
  cmdSetBaudRate[2] = 0x2F;
  cmdSetBaudRate[3] = 0x01;
  cmdSetBaudRate[4] = 0x00;
  cmdSetBaudRate[5] = 0x00;
  
  cmdSnapShot[0] = 0xAA;
  cmdSnapShot[1] = 0x05;
  cmdSnapShot[2] = 0x00;
  cmdSnapShot[3] = 0x00;
  cmdSnapShot[4] = 0x00;
  cmdSnapShot[5] = 0x00;
  
  cmdGetPic[0] = 0xAA;
  cmdGetPic[1] = 0x04;
  cmdGetPic[2] = 0x01;  // snap mode
  cmdGetPic[3] = 0x00;
  cmdGetPic[4] = 0x00;
  cmdGetPic[5] = 0x00;
  
  cmdEndPic[0] = 0xAA;
  cmdEndPic[1] = 0x0E;
  cmdEndPic[2] = 0x00;
  cmdEndPic[3] = 0x00;
  cmdEndPic[4] = 0xF0;
  cmdEndPic[5] = 0xF0;
}

/******************* send_cmd() ************************/
void send_cmd(unsigned char c[])
{
  int i;
  // send a command
  for(i = 0; i < cmdSize; i++)
  {
	  while (!(IFG1 & UTXIFG0));	  // USART0 TX buffer ready?
	  TXBUF0 = c[i];		
  } 
}

/*********************** sendSYNC () *********************/
void sendSYNC(void)
{
  // This loop will keep sending SYNC command to the camera
  // until the camera sends back ACK and SYNC command.
	 while (!finishSYNCflag)	
	 {
		int i = 0;
		i++;
		if (i > maxSendSYNC)
		{
			 send_cmd(cmdReset);
			 send_cmd(cmdPowerOff);
		}
		send_cmd(cmdSYNC);
		wait_a_sec(5);	
		if (currentLen == cmdACKsize) // receive complete message
		{
			currentLen = 0;		 
			finishSYNCflag = 1;
		}
	 }
}

/*********************** cam_prep () *********************/
void cam_prep (void) 
{
  
	while(!prepSYNCsuccess)
	{
	  sendSYNC();
	  wait_a_sec(5);
	  // The system sends back ACK command to camera
	  send_cmd(cmdACK);
	 
	  wait();
	  check();
	}
	 // send initial command and wait for ACK
	 send_cmd(cmdInitial);
	 while (!initialACKflag)
	 {
		 if (currentLen == cmdSize)
		 {
			 currentLen = 0;
			 initialACKflag = 1;
		 }
	 }
	 wait_a_sec(5);
	 
	 // send set package size command and wait for ACK
	 send_cmd(cmdSetPackageSize);
	 while (!setPackACKflag)
	 {
		 if (currentLen == cmdSize)
		 {
			 currentLen = 0;
			 setPackACKflag = 1;
		 }
	 }
	  wait_a_sec(5);	  
	  blinkLED(4);
}

/*********************** take_pic () *********************/
void take_pic(void)
{
	 // send snapshot command and wait for ACK
	 send_cmd(cmdSnapShot);
	 while (!snapACKflag)
	 {
		 if (currentLen == cmdSize)
		 {
	 currentLen = 0;
	 snapACKflag = 1;
		 }
	 }
	 wait_a_sec(5);
	 
	 // send get picture command and wait for ACK
	 send_cmd(cmdGetPic);
	 while (!getPicACKflag)
	 {	  
		 if (currentLen == cmdACKsize)
		 {
	 currentLen = 0;
	 getPicACKflag = 1;
		 }
	 }
	 cal_numbOfpack();
}

/*********************** get_image () *********************/
void get_image(void)
{

  dataReady = 1;	// start receiving data and put them into dummybuffer[] 
  currentLen = 0;
  
  unsigned short int packID = 0x00;
  // get each package from camera and dump it onto SD 
  while (packID < numberOfpackage)
  {
	 get_package(packID);
	 wait();
	 if (packID != numberOfpackage - 1)  // if packID is not the last packet
	 {
		prep_buffer();
	 }
	 else 
	 {
		prep_last_buffer();
	 }
	 packID++;
	 currentLen = 0;
	 dummyLen = 0;	
	 dummyRecv = 0;
  }
  wait();
  send_cmd(cmdEndPic);
  dataReady = 0;	 // stop storing into dummybuffer[]
}

/*********************** get_package () *********************/
void get_package(unsigned char id)
{
  int i;
  unsigned char cmd[6];
  cmd[0] = 0xAA;
  cmd[1] = 0x0E;
  cmd[2] = 0X0A;
  cmd[3] = 0X00;
  cmd[4] = id;
  cmd[5] = 0x00;
  
  // send a command
  for(i = 0; i < cmdSize; i++)
  {
	  while (!(IFG1 & UTXIFG0));	  // USART0 TX buffer ready?
	  TXBUF0 = cmd[i];		
  } 
}

/*************** prep_buffer() **************/
void prep_buffer(void)
{
  int i;
  if (bufFlag == 0)
  {
	 clear_mmc_buffer();
	 for (i = 4; i<260 ; i++)
		mmc_buffer[i-4] = dummybuffer[i];
	 bufFlag = 1;
  }
  else 
  {
	 for (i = 4 ; i < 260; i++)
		mmc_buffer[i+252] = dummybuffer[i];
	 transfer_pack();
	 blinkLED(2); 
	 bufFlag = 0;
  } 
}

/*************** prep_last_buffer() **************/
void prep_last_buffer(void)
{
  int i;
  cal_lastPackSize();	  // get last package size
  if (bufFlag == 0)
  {
	 clear_mmc_buffer();
	 for (i = 4; i < (lastPackDataSize + 4); i++)
		mmc_buffer[i-4] = dummybuffer[i];
	 transfer_pack();
	 blinkLED(2);
  }
  else 
  {
	 for (i = 4 ; i < (lastPackDataSize + 4); i++)
		mmc_buffer[i+252] = dummybuffer[i];
	 transfer_pack();
	 blinkLED(2);
  }  
}
/*************** transfer_pack() **************/
void transfer_pack(void)
{		
	 wait_a();
	 mmcWriteBlock(512*(block++));	 // write to SD card
	 wait_a();
}

/*************** clear_mmc_buffer() **************/
void clear_mmc_buffer(void)
{
  int i;
  for (i = 0 ; i < 512 ; i++)
	 mmc_buffer[i] = 0x00;
}

/*************** check() *******************/
void check(void)
{
  if (recvbuffer[1] == 0x0F || recvbuffer[7] == 0x0F)
  {
	 send_cmd(cmdReset);
	 send_cmd(cmdPowerOff);
  }
  else prepSYNCsuccess = 1;
}

/*************** cal_numbOfpack() *******************/
void cal_numbOfpack(void)
{
  unsigned short int totalBytes  = 0 ;  
  totalBytes = hex2int (recvbuffer[9], recvbuffer[10]); 
  picSize[numOfPic++] = totalBytes;
  numberOfpackage = (totalBytes / (packageSize - 6)) + 1;
}

/*************** cal_lastPackSize() **************/
void cal_lastPackSize(void)
{	
  lastPackDataSize = hex2int (recvbuffer[2], recvbuffer[3]);	 
}

/*************** hex2int() **********************/
unsigned short int hex2int(unsigned char c1, unsigned char c2)
{
  
  unsigned short int i1,i2,i3,i4; 
//  unsigned short int i5,i6; 
  unsigned short int total  = 0 ;
  
  i1 = (c1 & 0x0F);
  i2 = ((c1 & 0xF0) >> 4) * 16;
  i3 = (c2 & 0X0F) * 256;
  i4 = ((c2 & 0xF0) >> 4) * 4096;
  
//  i5 = (c3 & 0X0F) * 65536;
//  i6 = ((c3 & 0xF0) >> 4) * 1048576;
//  totalBytes = i1+i2+i3+i4+i5+i6;
  total = i1+i2+i3+i4;
  return total;
}
/**************** blinkLED() *******************/
void blinkLED(unsigned short int i)
{
  switch (i)
  {
  case 0:
	 LEDOn();
	 waitsec(2);
	 LEDOff();
	 waitsec(2);
	 break;
  case 1:
	 LED1On();
	 waitsec(2);
	 LED1Off();
	 waitsec(2);
	 break;
  case 2:
	 LED2On();
	 waitsec(2);
	 LED2Off();
	 waitsec(2);
	 break;
  case 3:
	 LED3On();
	 waitsec(2);
	 LED3Off();
	 waitsec(2);
	 break;
  case 4:
	 LED4On();
	 waitsec(2);
	 LED4Off();
	 waitsec(2);
	 break;
  default:
	 break;
  }
}

/*************** reset_sys() *******************/
void reset_sys(void)
{
  int i;

  timeflag_a = 0;
  timeflag_b = 0;
  tsec = 0;	
  dataReady = 0;
  bufFlag = 0;
  timeoutFlag = 0;
  dummyLen = 0;
  dummyRecv = 0;
  
  finishSYNCflag = 0;
  prepSYNCsuccess = 0;
  initialACKflag = 0;
  setPackACKflag = 0;
  snapACKflag = 0;
  getPicACKflag = 0;
  dataPackflag = 0;
  currentLen = 0;
  numberOfpackage = 0;
  lastPackDataSize = 0;
	
  for (i = 0; i < 262; i++) {
	dummybuffer[i] = 0x00;
	recvbuffer[i] = 0x00;
  }
 
  for (i = 0; i < 512; i++)
	 mmc_buffer[i] = 0x00;
}

unsigned int initEM260(void)
{
	/* Buffer for storing commands and responses */
	unsigned char buff[10];

	/* Reset the EM260 (P4.0 = 0). */
	P4OUT &= ~0x01;

	/* Wait for a while. */
	waitsec(10);

	/* Release the reset pin. */
	P4OUT |= 0x01;

	/* Wait for EM260 to assert nHOST_INT */
	while(!nHOST_INT);
	nHOST_INT = 0x00;

	/* Now, we must transmit a specific series of commands to finish
	 * initializing the EM260.		The first command is the "version"
	 * command.	  The first step is to load the "version" command (0x0A)
	 * and the frame terminator (0xA7) into the buffer.
	 */
	buff[0] = 0x0A;
	buff[1] = 0xA7;

	/* Assert nSSEL. */
	P3OUT &= ~0x01;

	/* Send the command to the EM260. */
	toEM260(buff, 2);

	/* Clear enough buffer space to receive the response. */
	memset(buff, 0, 3);

	/* Now, receive the response. */
	fromEM260(buff);

	/* Deassert the nSSEL signal. */
	P3OUT |= 0x01;

	/* If the first byte is not 0x00 and the second byte is not 0x02,
	 * there's a problem.
	 */
	if(buff[0] != 0x00 || buff[1] != 0x02)
		return 1;

	/* We must wait at least 1ms between commands. */
	wait_a();

	/* We must again transmit the "version" command. */
	buff[0] = 0x0A;
	buff[1] = 0xA7;
	P3OUT &= ~0x01;
	toEM260(buff, 2);
	memset(buff, 0, 3);
	fromEM260(buff);
	P3OUT |= 0x01;

	/* If the first byte of our response is not 0x82, there is
	 * a problem.
	 */
	if(buff[0] != 0x82)
		return 2;
	
	/* Again, we must delay for at least 1ms between commands. */
	wait_a();

	/* Now, transmit the "SPI status" command. */
	buff[0] = 0x0B;
	buff[1] = 0xA7;
	P3OUT &= ~0x01;
	toEM260(buff, 2);
	memset(buff, 0, 3);
	fromEM260(buff);
	P3OUT |= 0x01;

	/* If the first byte of the response is not 0xC1, there is
	 * a problem.
	 */
	if(buff[0] != 0xC1)
		return 3;
	
	/* Wait for at least 1ms. */
	wait_a();

	/* Finally, we must send the EZSP "version" command. */
	buff[0] = 0xFE;
	buff[1] = 0x04;
	buff[2] = 0x00;
	buff[3] = 0x00;
	buff[4] = 0x00;
	buff[5] = 0x02;
	buff[6] = 0xA7;
	P3OUT &= ~0x01;
	toEM260(buff, 7);
	memset(buff, 0, 10);
	fromEM260(buff);
	P3OUT |= 0x01;

	/* If bytes 5 and 6 are not each 0x02, something is wrong. */
	if(buff[5] != 0x02 || buff[6] != 0x02)
		return 4;
	
	/* Return successfully. */
	return 0;
}

void toEM260(unsigned char* buff, unsigned int len)
{
	/* Temporary receive buffer. */
	unsigned char tmp;

	/* Loop variable */
	unsigned int i;

	for(i = 0 ; i < len; i++)
	{
		/* Wait until the Tx buffer is empty. */
		while(!(IFG1 & UTXIFG0));

		/* Transmit the next byte in the buffer. */
		U0TXBUF = *(buff+i);

		/* Receive the idle response. */
		while(!(IFG1 & URXIFG0));
		tmp = U0RXBUF;
	}
}

unsigned int fromEM260(unsigned char* buff)
{
	/* Whether we have an EZSP frame or not */
	unsigned char ezsp;

	/* Length of the message we're receiving */
	unsigned char len;

	/* Loop variable */
	unsigned char i;

	/* We initialize i to 2 to account for the
	 * first two bytes that we receive
	 */
	i = 0x02;
	ezsp = 0;
	len = 0;

	/* Transmit 0xFF and receive 0xFF to clock the SPI.  Do so until
	 * we receive a byte that is not 0xFF.  This byte is the first
	 * byte of the response.
	 */
	*buff = 0xFF;
	while(*buff == 0xFF)
	{
		while(!(IFG1 & UTXIFG0));
		U0TXBUF = 0xFF;

		while(!(IFG1 & URXIFG0));
		*buff = U0RXBUF;
	}

	/* If the first bytes is 0xFE, we're receiving an EZSP frame. */
	if (*buff == 0xFE)
	{
		ezsp = 1;
	}

	/* Again, transmit 0xFF and receive the next byte of the
	 * message.
	 */
	while(!(IFG1 & UTXIFG0));
	U0TXBUF = 0xFF;
	while(!(IFG1 & URXIFG0));
	*(buff+1) = U0RXBUF;

	/* Some messages are only two bytes.  If the second byte is
	 * 0xA7 (the frame terminator), then we have such a message
	 * and there is no need to continue.
	 */
	if(*(buff+1) == 0xA7)
	{
		return i;
	}

	/* If we're receiving an EZSP frame, the length of the message
	 * is the second byte that we received.  Otherwise, the length
	 * of the message must be three bytes.  We always add three to
	 * the length of the message to account for the initial byte,
	 * the second byte, and the frame terminator.
	 */
	len = ezsp ? *(buff+1) : 0;
	len += 3;

	/* Now, transmit 0xFF to clock the SPI and receive the next
	 * byte of the response.	Do so until we have the entire
	 * response.
	 */
	do
	{
		/* Transmit 0xFF to the EM260. */
		while(!(IFG1 & UTXIFG0));
		U0TXBUF = 0xFF;

		/* Receive a response. */
		while(!(IFG1 & URXIFG0));
		*(buff+i) = U0RXBUF;

		++i;
	} while(i < len);

	/* Return the number of bytes that we received. */
	return i;
}

unsigned char EM260_setConfigurationValue(unsigned char* params)
{
	/* Command buffer */
	unsigned char buff[9];

	/* Set up the command buffer. */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x06;		/* Length of the EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x53;		/* EZSP: setConfigurationValue() command */
	buff[5] = params[0];	/* EZSP: Param: EZSP ConfigID */
	buff[6] = params[1];	/* EZSP: Param: new value (LSB) */
	buff[7] = params[2];	/* EZSP: Param: new value (MSB) */
	buff[8] = 0xA7;		/* Frame terminator */

	/* Transmit the command and receive a response. */
	P3OUT &= ~0x01;
	toEM260(buff, 9);
	memset(buff, 0, 9);
	fromEM260(buff);
	nHOST_INT = 0;
	P3OUT |= 0x01;

	/* Return the result of the command. */
	return buff[5];
}

void EM260_callback(unsigned char* buff)
{
	/* Set up the command buffer */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x06;		/* EZSP: callback() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send the command and receive a response */
	P3OUT &= ~0x01;
	toEM260(buff, 6);
	memset(buff, 0, 6);
	fromEM260(buff);
	nHOST_INT = 0;
	P3OUT |= 0x01;
}

void EM260_networkInit()
{
	/* Command buffer */
	unsigned char buff[7];

	/*Set up command buffer */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of the EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x17;		/* EZSP: networkInit() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send a command and receive a response */
	P3OUT &= ~0x01;
	toEM260(buff, 6);
	memset(buff, 0, 6);
	fromEM260(buff);
	nHOST_INT = 0x00;
	P3OUT |= 0x01;

	/* If we're on a network, there will be a callback. */
	if(buff[5] != 0x93)
	{
		waitsec(10);
		while(!nHOST_INT);
		EM260_callback(buff);

		waitsec(10);
		EM260_leaveNetwork();
		waitsec(10);
		while(!nHOST_INT);
		EM260_callback(buff);
	}
}

unsigned char EM260_leaveNetwork()
{
	/* Command buffer */
	unsigned char buff[7];

	/*Set up command buffer */ 
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of the EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x20;		/* EZSP: leaveNetwork() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send a command and receive a response */
	P3OUT &= ~0x01;
	toEM260(buff, 6);
	memset(buff, 0, 6);
	fromEM260(buff);
	nHOST_INT = 0;
	P3OUT |= 0x01;

	/* Return the result of the command. */
	return buff[5];
}

unsigned char EM260_scanNetwork()
{
	/* Command buffer */
	unsigned char buff[12];

	/* Set up the command buffer */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x09;		/* Length of EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x1A;		/* EZSP: startScan() function */
	buff[5] = 0x01;		/* EZSP: Param: EmberNetworkScanType */
	buff[6] = 0x00;		/* EZSP: Param: channel mask (LSB) */
	buff[7] = 0xF8;		/* EZSP: Param: channel mask */
	buff[8] = 0xFF;		/* EZSP: Param: channel mask */
	buff[9] = 0x07;		/* EZSP: Param: channel mask (MSB) */
	buff[10]= 0x02;		/* EZSP: Param: scan duration exponent */
	buff[11] = 0xA7;	/* Frame terminator */

	/* Send the command and receive a response */
	P3OUT &= ~0x01;
	toEM260(buff, 12);
	memset(buff, 0, 12);
	fromEM260(buff);
	nHOST_INT = 0;
	P3OUT |= 0x01;

	/* Return the result of the command. */
	return buff[5];
}

unsigned char EM260_justJoinNetwork(unsigned char* param)
{
	/* Command buffer */
	unsigned char buff[19];

	/* Set up the command buffer */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x10;		/* Length of EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x1F;		/* EZSP: joinNetwork() command */
	buff[5] = 0x02;		/* EZSP: Param: node type (router) */
	buff[6] = param[8];	/* EZSP: Param: extended PAN ID (LSB) */
	buff[7] = param[9];	/* EZSP: Param: extended PAN ID */
	buff[8] = param[10];	/* EZSP: Param: extended PAN ID */
	buff[9] = param[11];	/* EZSP: Param: extended PAN ID */
	buff[10] = param[12];	/* EZSP: Param: extended PAN ID */
	buff[11] = param[13];	/* EZSP: Param: extended PAN ID */
	buff[12] = param[14];	/* EZSP: Param: extended PAN ID */
	buff[13] = param[15];	/* EZSP: Param: extended PAN ID (MSB) */
	buff[14] = param[6];	/* EZSP: Param: short PAN ID (LSB) */
	buff[15] = param[7];	/* EZSP: Param: short PAN ID (MSB) */
	buff[16] = 0x00;	/* EZSP: Param: radio Tx power (0 dBm) */
	buff[17] = param[5];	/* EZSP: Param: radio channel */
	buff[18] = 0xA7;	/* Frame terminator */

	/* Send the command and receive a response */
	P3OUT &= ~0x01;
	toEM260(buff, 19);
	memset(buff, 0, 19);
	fromEM260(buff);
	nHOST_INT = 0;
	P3OUT |= 0x01;

	/* Return the result of the command. */
	return buff[5];
}

void EM260_getNodeID(unsigned char* buff)
{
	/* Set up the command buffer */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x27;		/* EZSP: getNodeID() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send the command and receive a response */
	P3OUT &= ~0x01;
	toEM260(buff, 6);
	memset(buff, 0, 6);
	fromEM260(buff);
	nHOST_INT = 0;
	P3OUT |= 0x01;
}

unsigned char EM260_sendUnicast(unsigned char* message, unsigned char* nodeID,
	unsigned char len)
{
	/* Command buffer */
	unsigned char command[24+len];

	/* Loop variable */
	unsigned int i;
	i = 0;

	/* Set up the command buffer */
	command[0] = 0xFE;	/* EZSP frame */
	command[1] = 21 + len;	/* Length of the EZSP frame */
	command[2] = sq++;	/* EZSP: sequence number */
	command[3] = 0x00;	/* EZSP: command with no sleeping */
	command[4] = 0x34;	/* EZSP: sendUnicast() command */
	command[5] = 0x00;	/* EZSP: Param: MessageType (direct) */
	command[6] = 0x00;	/* EZSP: Param: destination NodeID (LSB) */
	command[7] = 0x00;	/* EZSP: Param: destination NodeID (MSB) */
	command[8] = 0x00;	/* EZSP: Param: profileID (LSB) */
	command[9] = 0x00;	/* EZSP: Param: profileID (MSB) */
	command[10] = 0x00;	/* EZSP: Param: clusterID (LSB) */
	command[11] = 0x00;	/* EZSP: Param: clusterID (MSB) */
	command[12] = 0x01;	/* EZSP: Param: source EndPoint */
	command[13] = 0x01;	/* EZSP: Param: destination EndPoint */
	command[14] = 0x40;	/* EZSP: Param: APS options (LSB) */
	command[15] = 0x05;	/* EZSP: Param: APS options (MSB) */
	command[16] = 0x00;	/* EZSP: Param: groupID (LSB) */
	command[17] = 0x00;	/* EZSP: Param: groupID (MSB) */
	command[18] = sq2++;	/* EZSP: Param: message sequence number */
	command[19] = 0x64;	/* EZSP: Param: message tag */
	command[20] = len + 2;	/* EZSP: Param: length of the message */

	/* Now we include the NodeID, followed by the actual message.
	 * Including the NodeID makes it easier for the base station
	 * to reply to us.
	 */
	command[21] = nodeID[0];
	command[22] = nodeID[1];
	for(i = 0; i < len; i++)
	{
		command[i+23] = message[i];
	}
	command[23+len] = 0xA7;	/* Frame terminator */

	/* Send the command and receive a response. */
	P3OUT &= ~0x01;
	toEM260(command, 24+len);
	memset(command, 0, 24+len);
	fromEM260(command);
	nHOST_INT = 0;
	P3OUT |= 0x01;

	/* Return the result of the command. */
	return command[5];
}

unsigned char EM260_networkState()
{
	/* Command buffer */
	unsigned char buff[7];

	/* Set up the command buffer */
	buff[0] = 0xFE;		/* EZSP frame */
	buff[1] = 0x03;		/* Length of the EZSP frame */
	buff[2] = sq++;		/* EZSP: sequence number */
	buff[3] = 0x00;		/* EZSP: command with no sleeping */
	buff[4] = 0x18;		/* EZSP: networkState() command */
	buff[5] = 0xA7;		/* Frame terminator */

	/* Send the command and receive a response. */
	P3OUT &= ~0x01;
	toEM260(buff, 6);
	memset(buff, 0, 6);
	fromEM260(buff);
	nHOST_INT = 0;
	P3OUT |= 0x01;
	
	/* Return the result of the command. */
	return buff[5];
}

void joinNetwork()
{
	/* Command buffers */
	unsigned char buff1[136];

	/* Status */
	unsigned char stat;

	/* We must first call getNetworkState() */
	stat = EM260_networkState();

	/* If we are already on a network, we must leave it. */
	if(stat == 0x02 || stat == 0x03)
	{
		//waitsec(10);
		//while(!nHOST_INT);
		//EM260_callback(buff1);
		//memset(buff1, 0, 136);
		//waitsec(10);
		EM260_leaveNetwork();
		waitsec(10);
		while(!nHOST_INT);
		EM260_callback(buff1);
		memset(buff1, 0, 136);
	}

	/* Turn off the network indicator LED. */
	P6OUT |= 0x04;
	
	/* Find a new network to join. */
	buff1[4] = 0x00;
	while (buff1[4] != 0x1B)
	{
		memset(buff1, 0, 136);
		stat = 0xFF;
		while (stat != 0x00)
		{
			waitsec(10);
			stat = EM260_scanNetwork();
			blinkLED(3);
		}

		waitsec(10);
		EM260_callback(buff1);
	}
	
	/* Use the parameters found in scanNetwork() to join
	 * a new network.
	 */
	stat = 0xFF;
	while (stat != 0x00)
	{
		waitsec(10);
		stat = EM260_justJoinNetwork(buff1);
	}
	
	/* Because the order of callbacks is not always predictable,
	 * we might not get the joinNetwork() callback first.  So,
	 * we'll keep sending callbacks until we do get it.
	 */
	memset(buff1, 0, 136);
	while (buff1[5] != 0x90)
	{
		waitsec(10);
		memset(buff1, 0, 136);
		EM260_callback(buff1);
	}
	
	waitsec(10);
	memset(buff1, 0, 136);

	/* Get our new NodeID. */
	EM260_getNodeID(buff1);
	nodeID[0] = buff1[5];
	nodeID[1] = buff1[6];
	
	/* Turn on the network indicator LED. */
	P6OUT &= ~0x04;
}

void transmitPicture(unsigned int imgNum)
{
	/* Message buffer */
	unsigned char buff1[136];

	/* Number of packets to send */
	unsigned int packets;

	/* Result of EM260 commands */
	unsigned char status;

	/* Number of bytes not yet sent */
	unsigned int bytesRemaining;

	/* Used in packet calculations */
	unsigned int tmp;

	/* Loop variables */
	unsigned int i;
	unsigned int j;
	unsigned int k;

	/* First, we must calculate the number of packets to send. */
	packets = picSize[imgNum] / 64;
	if(packets * 64 < picSize[imgNum])
		++packets;
	
	/* Now, we must convert the number of packets into readable hex. */
	buff1[13] = packets / 4096;
	tmp = packets % 4096;
	buff1[14] = tmp / 256;
	packets = tmp % 256;
	buff1[15] = packets / 16;
	buff1[16] = packets % 16;

	/* We must now send the IMAGE_READY0xNNNN signal. */
	buff1[0] = 'I';
	buff1[1] = 'M';
	buff1[2] = 'A';
	buff1[3] = 'G';
	buff1[4] = 'E';
	buff1[5] = '_';
	buff1[6] = 'R';
	buff1[7] = 'E';
	buff1[8] = 'A';
	buff1[9] = 'D';
	buff1[10] = 'Y';
	buff1[11] = '0';
	buff1[12] = 'x';
	buff1[13] += (buff1[13] > 9 ? 55 : 48);
	buff1[14] += (buff1[14] > 9 ? 55 : 48);
	buff1[15] += (buff1[15] > 9 ? 55 : 48);
	buff1[16] += (buff1[16] > 9 ? 55 : 48);

	status = EM260_sendUnicast(buff1, nodeID, 19);
	while(status != 0x00)
	{
		wait_a();
		joinNetwork();
		wait_a();
		status = EM260_sendUnicast(buff1, nodeID, 19);
	}

	/* Wait for the message sent callback. */
	memset(buff1, 0, 136);
	wait_a();
	while(!nHOST_INT);
	EM260_callback(buff1);
	memset(buff1, 0, 136);

	/* Now, wait for the base station to send the IMAGE_READY_OK
	 * signal.  The first callback will get a generic ACK from the
	 * base station and the second will get the IMAGE_READY_OK signal.
	 */
	while(!nHOST_INT);
	while(buff1[4] != 0x45)
	{
		memset(buff1, 0, 136);
		EM260_callback(buff1);
		wait_a();
	}
	memset(buff1, 0, 136);
	while(buff1[4] != 0x45)
	{
		memset(buff1, 0, 136);
		EM260_callback(buff1);
		wait_a();
	}

	/* Now, we can actually send the image to the base station.  We do
	 * so by reading the image data from the SD card in blocks of 512
	 * bytes and sending it in packets of 64 bytes.
	 */
	bytesRemaining = picSize[imgNum];
	while(bytesRemaining)
	{
		mmcReadBlock((512*numBlocks++), 512);
		for(i = 0; i < 512; i++)
		{
			memset(buff1, 0, 136);
			k = (bytesRemaining >= 64 ? 64 : bytesRemaining);
			buff1[0] = k / 16 + ((k / 16) > 9 ? 55 : 48);
			buff1[1] = k % 16 + ((k % 16) > 9 ? 55 : 48);
			buff1[2] = ',';
			for(j = 0; j < k; j++)
			{
				buff1[j+3] = mmc_buffer[i++];
			}
			--i;
			do
			{
				status = EM260_sendUnicast(buff1, nodeID, k+5);
				wait_a();
			} while(status != 0x00);

			memset(buff1, 0, 136);
			EM260_callback(buff1);
			memset(buff1, 0, 136);
			wait_a();
			bytesRemaining -= k;
			do
			{
				memset(buff1, 0, 136);
				EM260_callback(buff1);
				wait_a();
			} while(buff1[4] != 0x45);

			if(bytesRemaining == 0)
				break;
		}
	}
}
